//
//  main.c
//  Example for SMBuffer
//
//  Created by Julien-Pierre Avrous on 22/08/06.
//  Copyright 2006 SourceMac.com. All rights reserved.
//

#include <iostream> //printf etc
#include <pthread.h> //thread
#include <unistd.h> //usleep
#include <stdlib.h> //random
#include <stdarg.h> //... of myprintf

#include "SMBuffer.h" //buffer

static bool stop=false; //just for the main loop, but useless here

void *inputThread(void* buf);
void *outputThread(void* buf);
void myprintf(const char *format, ...);

#define MAX_RLONG 0x7FFFFFFF

using namespace std;

int main()
{
	pthread_t inThread, outThread; // Dclaration des threads
	SMBuffer *buffer=new SMBuffer; // Dclaration + instanciation du buffer
		
	srandomdev(); // init randomizer
	
	cout << "Averous Julien-Pierre - 2006 - www.sourcemac.com" << endl;

	if(pthread_create(&inThread, NULL, inputThread, (void*)buffer))
		return -1; //error creating thread
	if(pthread_create(&outThread, NULL, outputThread, (void*)buffer))
		return -1; //error creating thread

	// loop for maintain our sample app in life. In fact, in this example, "stop" is useless (you should quit the app with ctrl+c)
	while(!stop) usleep(50000);
	
	
	delete buffer; // clean the content buffer and the instance
	
	return 0;
}



/* a loop function to fill the buffer, datas from a socket by example */
void *inputThread(void* buf)
{
	SMBuffer *buffer=(SMBuffer*)buf; // we retrieve the buffer
	char print_buffer[255]; // because of the multi-threading, we need to concatenate the characters to show them all at once.
	char *dt;
	int i;
	int rnd;
	while(!stop)
		{
		/* generate random data to fill the buffer */
				
		rnd=((float)random()/(float)MAX_RLONG)*50; // we generate a random size of data block
		if(rnd)
			{
			dt=(char*)malloc(rnd);
			for(i=0; i<rnd; i++)
				{
				char rc='a'+((float)random()/(float)MAX_RLONG)*26; // we generate random characters
				dt[i]=rc;
				print_buffer[i]=rc;
				}
			print_buffer[i]=0;
			
			myprintf("Input => Data goes in the buffer (%i) : %s\n", rnd, print_buffer); // Print to follow the running
			buffer->addBufferNoCopy(dt, rnd); // we set the data in the buffer without copy because we have already a malloc buffer
			}
		
		/* not too fast */
		usleep(1000000);
		}
	pthread_exit(NULL);
}


/* a loop function to empty the buffer, for decodind data by example */
void *outputThread(void* buf)
{
	SMBuffer *buffer=(SMBuffer*)buf;
	char cbuffer[100], *bbuffer;
	char print_buffer[255];
	unsigned int i, cnt;

	while(!stop)
		{
		while(buffer->bufferSize()<100) // we want a minimum size for our buffer (eg an audio decoder who need a minimum buffer size)
			{
			myprintf("Output => Waiting for data ...\n");
			usleep(500000);
			}
			
		cnt=buffer->copyContinuousRead((void*)cbuffer, 100); // we copy da from the buffer to a local buffer and set a 0 ending to print it (the used data is liberated)
		for(i=0; i<100; i++)
			print_buffer[i]=cbuffer[i];
		print_buffer[i]=0;
		myprintf("Output => Get a continous raw data (%i) : %s\n", cnt, print_buffer); // show the recovered data
		
		myprintf("Output => Buffer size : %i\n", buffer->bufferSize()); // show the buffer size after the copyContinuousRead
		
		bbuffer=NULL;
		buffer->getBuffer((void**)&bbuffer, &cnt); // get the residual frame cuted by copyContinuousRead, if there is one (note : here is useless, is just for the example !)
		if(bbuffer)
			{
			for(i=0; i<cnt; i++)
				print_buffer[i]=bbuffer[i];
			print_buffer[i]=0;
			myprintf("Output => Get rest data in the last block data at the end of the buffer (%i) : %s\n", cnt, print_buffer);
			free(bbuffer);
			}
		myprintf("Output => Buffer size : %i\n", buffer->bufferSize());
		usleep(50000);
		}

	pthread_exit(NULL);
}


void myprintf(const char *format, ...)
{	
	va_list ap;
	va_start(ap, format);
	
	vprintf(format, ap);
	fflush(stdout);
	
	va_end(ap);
}